function [B dB] = BB2dDeg2L(v,x,type)
%    [B dB]= BB2dDeg2L(v,x,type)
% Evaluate generalized quadratic BB functions
% and gradients in a convex polygon
%
% Inputs:
% v    : [x1 y1; x2 y2; ...; xn yn], the n vertices of the polygon, ccw
% x    : [x(1) x(2)], the point at which the functions are evaluated
% type : type of barycentric coordinate:
%        type=1 => Wachspress
%        type=2 => MV local
%        type=3 => bilinear, if n=4.
%        type=4 => family.
% Outputs:
% B    : output basis functions = [B_11; B_12; B_22; ...; B_2n+1]
% dB   : output gradient of basis functions = [dB_11; dB_12; ...; dB_2n+1]
%
% Author: Michael Floater
% e-mail: michaelf@ifi.uio.no
% Created: Mar 2014, using Matlab R2013b
% Modified: Nov 2014, gradients added
% Copyright 2014

  n = size(v,1);
  B = zeros(2*n,1);

  if nargout == 2
    dB = zeros(2*n,2);
    [phi dphi] = bary2dL(v,x,type);
  else
    phi = bary2dL(v,x,type);
  end
  
  lamiCell = zeros(3,n);
  dlamiCell = zeros(3,2,n);
  for i=1:n
    im1 = mod1(i-1,n);
    ip1 = mod1(i+1,n);
    vi = zeros(3,2);
    vi(1,:) = v(im1,:);
    vi(2,:) = v(i,:);
    vi(3,:) = v(ip1,:);
    if nargout == 2
        [lamiCell(:,i) dlamiCell(:,:,i)] = baryTriangL(vi,x);
      else
        lamiCell(:,i) = baryTriangL(vi,x);
    end
  end
  
  for i=1:n
    ip1 = mod1(i+1,n);
    B(2*i-1) = phi(i) * lamiCell(2,i);
    B(2*i) = phi(i) * lamiCell(3,i) + phi(ip1) * lamiCell(1,ip1);
    
    if nargout == 2
      dB(2*i-1,:) = dphi(i,:) * lamiCell(2,i) + phi(i) * dlamiCell(2,:,i);
      dB(2*i,:) = dphi(i,:) * lamiCell(3,i) + phi(i) * dlamiCell(3,:,i) +... 
                dphi(ip1,:) * lamiCell(1,ip1) + phi(ip1) * dlamiCell(1,:,ip1);
    end
  end

