%This is a matlab demo program to show how to compute a numerical 
%solution of Poisson equation using serendipity quadratic finite elements
%based on Generalized Barycentric Coordinates. 
%This is written by Dr. Ming-Jun Lai and Michael Floater on Dec. 3, 2014.
%based on the results in the publication:
%  Polygonal spline spaces and  the numerical solution of the Poisson equation,
%    Michael Floater and Ming-Jun Lai
%  SIAM J. Numerical Analysis, accepted on  2015.
%
%V=[0.7 0;1.5 -0.4;2.2 0;3 1;3 2;2 3;1 3;0 2.2;-0.2 1.5;0 0.8]/3; P={[1 2 3 4 5 6 7 8 9 10]};
%V=[0.7 0;1.5 -0.4;2.2 0;3 1;3 2;2 3;1 3;0 2;0 1]/3; P={[1 2 3 4 5 6 7 8 9]};
%V=[1 0;2 0;3 1;3 2;2 3;1 3;0 2;0 1]/3; P={[1 2 3 4 5 6 7 8]}; %Octagon
%V=[1 0;2 0;3 1;3 2;2 3;1 3;0 2]/3; P={[1 2 3 4 5 6 7]}; %Heptagon
%V=[1 0;2 1;2 2;1 3;0 2;0 1]; V(:,2)=V(:,2)/3; V(:,1)=V(:,1)/2; P={[1 2 3 4 5 6]}; %Hexagon
%V=[1 0;2 0;2 1;1 1;1 2;0 2;0 1;0.5 0.5]; P={[1 2 3 4 8],[8 4 5 6 7]}; %Pentagon
%V=[1 0;1.4 0;2 0;2 1;1 1;1 2;0 2;0 1]; P={[1 2 5 8],[2 3 4 5],[5 6 7 8]}; %Quadrilateral
%V=[0 0;1 0;1 1;0 1]; P={[1 2 3],[1 3 4]}; [V,P]=refine(V,P); %triangulation
V=[0 0;1 0;2 0;3 0;3 1;3 2;3 3;2 3;0 3;0 1;1 1;2 2]/3;
P={[1 2 11 10],[2 3 5 6 12 11],[6 7 8 12],[8 9 10 11 12],[3 4 5]}; %a Mixed Partition
%V=[0 0;1 0;2 0;3 1;2 2;1 2;1/2 2;0 2;1/2 1]/2; P={[2 3 4 5 6 9],[1 2 9],[6 7 9],[1 9 7 8]}; %ex1
plot_v(V,P)
d = 2;
CaseNumber=5; %number in the list of testing solutions. See g.m  for more solutions.
xmin=min(V(:,1));xmax=max(V(:,1));
ymin=min(V(:,2));ymax=max(V(:,2)); 
xstep=(xmax-xmin)/1000;ystep=(ymax-ymin)/1000;
[xx,yy]=meshgrid(xmin:xstep:xmax,ymin:ystep:ymax);
zz=g(xx,yy,CaseNumber);
errP=[]; L=4; err=[]; warning off
for test=1:L
[V,P]=refine(V,P); %refinement of polygonal mesh (V,P)
[E,PE,PV,EV,BV] = vdata(V,P);%relations of polygonal mesh (V,P)
H = c0continuity(P,E,PE,d); %continuous conditions
tic
[M,K]=msGBCvP(V,P); %mass and stiffness matrices generator
toc
F = coeffv(V,P,'laplace',CaseNumber); %the right-hand side term
[G,B]=dirichlet(V,P,E,PE,d,'g',CaseNumber); %Dirichlet boundary.
n = size(H,1);
k = length(G);
p = size(H,1);
ZERO = zeros(p,1);
tol=1.0e-6;
eps=1.0e-4;
max_it=5;
tic
c=CImin(K,[H;B],M*F,[ZERO;G],eps,max_it,tol); %constrained iterative minimization
toc
tic
zzGBC= gbcVevalPn(V,P,c,xx(:),yy(:)); %evaluation of polygonal spline solution.
toc
I=isnan(zzGBC); J=find(I==0); 
err1=[norm(zz(J)-zzGBC(J),inf),sqrt(mse(zz(J)-zzGBC(J)))],
err=[err;err1];
figure, surf(xx,yy,reshape(zzGBC,size(xx))), shading interp, hold on, plot_v(V,P)
end
err1=err(:,2); 
log(err1(1:L-1)./err1(2:L))/log(2) %convergence rate alpha, i.e. (1/2^alpha).
