function [phi dphi] = bary2dL(v,x,type)
%
% Evaluate barycentric coordinates and their gradients in a convex polygon
%
% Inputs:
% v    : [x1 y1; x2 y2; ...; xn yn], the n vertices of the polygon in ccw
% x    : [x(1) x(2)], the point at which the basis functions are computed
% type : type of barycentric coordinate:
%        type=1 => Wachspress
%        type=2 => MV local
%        type=3 => inverse bilinear, if n=4.
%        type=4 => inverse rational bilinear, if n=4.
%        type=5 => family.
% Outputs:
% phi  : output basis functions = [phi_1; ...; phi_n]
% dphi : output gradient of basis functions = [dphi_1; ...; dphi_n]
%
% Author: Michael Floater
% e-mail: michaelf@ifi.uio.no
% Created: Mar 2014, using Matlab R2013b
% Copyright 2014

  n = size(v,1);
  phi = zeros(n,1);
  dphi = zeros(n,2);
  if type == 1
    if nargout == 2
      [phi dphi] = wachspress2dL(v,x);
    else
      phi = wachspress2dL(v,x);
    end
  elseif type == 2
    [phi dphi] = mv2d(v,x);
  elseif type == 3 && n == 4
    [phi dphi] = invBilinear(v,x);
  elseif type == 4 && n == 4
    %w = makeRatWeights(v);
    %rho = w(1)*w(3)/(w(2)*w(4));
    w = [5 1 1 1];
    %[phi dphi] = invRatBilinear(v,w,x);
    [phi dphi] = invRatBilinear2(v,w,x);
  else
    [phi dphi] = family(v,x);
  end
