function [phi dxphi dyphi] = baryTriangP(v,x)
%
% Evaluate the three barycentric coordinates
% and their gradients in a triangle.
%
% Inputs:
% v    : [x1 y1; x2 y2; x3 y3], the 3 vertices of the triangle, ccw
% x    : [x(1) x(2)], the point at which the basis functions are computed
% Outputs:
% phi  : output coordinates = [phi_1; phi_2; phi_3]
% dphi : output gradients of the coordinates = [dphi_1; dphi_2; dphi_3]
%
% Authors: Michael Floater and Ming-Jun Lai
% e-mail: michaelf@ifi.uio.no and mjlai@uga.edu
% Created: Nov 2014, using Matlab R2014a
% Modified: March,2015 for multiple points.
% Copyright 2014
m=size(x,1);
phi  = zeros(3,m);
dxphi = zeros(3,m); dyphi=dxphi;
e = zeros(3,2);
e(1,:) = v(2,:)-v(1,:);
e(2,:) = v(3,:)-v(2,:);
e(3,:) = v(1,:)-v(3,:);
%C = det([e(1,:); e(2,:)]);
C=e(1,1)*e(2,2)-e(2,1)*e(1,2);
d1 = zeros(3,m); d2=zeros(3,m);
for i=1:3
  d1(i,:) = v(i,1)-x(:,1)';
  d2(i,:) = v(i,2)-x(:,2)';
end
%phi(1) = det([d(2,:); d(3,:)]) / C;
phi(1,:)= (d1(2,:).*d2(3,:) - d2(2,:).*d1(3,:))/C;
%phi(2) = det([d(3,:); d(1,:)]) / C;
phi(2,:)=(d1(3,:).*d2(1,:)- d2(3,:).*d1(1,:))/C;
%phi(3) = det([d(1,:); d(2,:)]) / C;
phi(3,:)=(d1(1,:).*d2(2,:)- d2(1,:).*d1(2,:))/C;
rot = inline('[-y(2), y(1)]','y');
e1=rot(e(2,:));
dxphi(1,:) = e1(1)*ones(1,m)/ C;
dyphi(1,:) = e1(2)*ones(1,m)/ C;
e2=rot(e(3,:));
dxphi(2,:) = e2(1)*ones(1,m)/ C;
dyphi(2,:) = e2(2)*ones(1,m)/ C;
e3=  rot(e(1,:));
dxphi(3,:) = e3(1)*ones(1,m) / C;
dyphi(3,:) = e3(2)*ones(1,m) / C;

