function Z= gbcVevalPn(V,P,c,X,Y);
%        Z = gbcPevalP(V,P,c,X,Y)
% This function evaluates the piecewise function over the polygons [V,P] defined 
% by  coefficient vector c at the points with coordinates X and Y, where
% Author: Ming-Jun Lai
% e-mail: mjlai@math.uga.edu
% Created: Dec. 2014, using Matlab R2014b
% Modified: March 2015 for multiple points in parallel computation.
% Copyright 2015.
[l,n] = size(P); type=1;
Z = nan*ones(size(X));
tol =100*eps; M1=zeros(n,1);
parfor j=1:n
 p=P{j}; [l,k]=size(p); m=2*k; M1(j)=m;
switch k
 case 3
%  if k==3
[lam1,lam2,lam3] = bary(V(p(1),:),V(p(2),:),V(p(3),:),X,Y);
   I = find(lam1 >=-tol & lam2 >= -tol & lam3 >= -tol);
   if ~isempty(I)
     L1=lam1(I); L2=lam2(I); L3=lam3(I);
     phi1=L1.^2-L1.*L2-L1.*L3;phi2=4*L1.*L2;phi3=L2.^2-L1.*L2-L2.*L3;
     phi4=4*L2.*L3; phi5=L3.^2-L1.*L3-L2.*L3; phi6=4*L1.*L3; 
     L=[phi1 phi2 phi3 phi4 phi5 phi6]; 
     M2{j}=L; S2{j}=I;
   end;
%  end
 case 4
%  if k==4
  [lam1,lam2,lam3,lam4] = baryQ(V(p(1),:),V(p(2),:),V(p(3),:),V(p(4),:),X,Y);
   I = find(lam1 >=-tol & lam2 >= -tol & lam3 >= -tol & lam4>=-tol);
   if ~isempty(I)
       v=[V(p(1),:);V(p(2),:);V(p(3),:);V(p(4),:)];
       xx=X(I);yy=Y(I); 
      L = mlag2dDeg2(v,[xx,yy],type);
           M2{j}=L; S2{j}=I;
%      Z(I)=L*c(N+1:N+m); N=N+m;
   end;
%  end
case 5
%  if k==5
[lam1,lam2,lam3,lam4,lam5] = baryP(V(p(1),:),V(p(2),:),V(p(3),:),V(p(4),:),V(p(5),:),X,Y);
I = find(lam1 >=-tol & lam2 >= -tol & lam3 >= -tol & lam4>=-tol & lam5>=-tol);
   if ~isempty(I)
       v=[V(p(1),:);V(p(2),:);V(p(3),:);V(p(4),:);V(p(5),:)];
       xx=X(I);yy=Y(I); 
      L = mlag2dDeg2(v,[xx,yy],type);
           M2{j}=L; S2{j}=I;
%      Z(I)=L*c(N+1:N+m); N=N+m;
   end;
%  end
%  if k==6
case 6
[lam1,lam2,lam3,lam4,lam5,lam6] = baryH(V(p(1),:),V(p(2),:),V(p(3),:),V(p(4),:),V(p(5),:),V(p(6),:),X,Y);
I = find(lam1 >=-tol & lam2 >= -tol & lam3 >= -tol & lam4>=-tol & lam5>=-tol &lam6>=-tol);
   if ~isempty(I)
       v=[V(p(1),:);V(p(2),:);V(p(3),:);V(p(4),:);V(p(5),:);V(p(6),:)];
       xx=X(I);yy=Y(I); 
      L = mlag2dDeg2(v,[xx,yy],type);
           M2{j}=L; S2{j}=I;
%      Z(I)=L*c(N+1:N+m); N=N+m;
   end;
%   end
case 7
%  if k==7
[lam1,lam2,lam3,lam4,lam5,lam6,lam7] = ... 
baryS(V(p(1),:),V(p(2),:),V(p(3),:),V(p(4),:),V(p(5),:),V(p(6),:),V(p(7),:),X,Y);
I = find(lam1 >=-tol & lam2 >= -tol & lam3 >= -tol & lam4>=-tol & lam5>=-tol &lam6>=-tol &lam7>=-tol);
   if ~isempty(I)
       v=[V(p(1),:);V(p(2),:);V(p(3),:);V(p(4),:);V(p(5),:);V(p(6),:);V(p(7),:)];
       xx=X(I);yy=Y(I); 
      L = mlag2dDeg2(v,[xx,yy],type);
           M2{j}=L; S2{j}=I;
%      Z(I)=L*c(N+1:N+m); N=N+m;
   end;
%  end
case 8
% if k==8
[lam1,lam2,lam3,lam4,lam5,lam6,lam7,lam8] = ... 
baryO(V(p(1),:),V(p(2),:),V(p(3),:),V(p(4),:),V(p(5),:),V(p(6),:),V(p(7),:),V(p(8),:),X,Y);
I = find(lam1>=-tol&lam2>= -tol&lam3>= -tol&lam4>=-tol&lam5>=-tol&lam6>=-tol &lam7>=-tol&lam8>=-tol);
   if ~isempty(I)
       v=[V(p(1),:);V(p(2),:);V(p(3),:);V(p(4),:);V(p(5),:);V(p(6),:);V(p(7),:);V(p(8),:)];
       xx=X(I);yy=Y(I); 
      L = mlag2dDeg2(v,[xx,yy],type);
           M2{j}=L; S2{j}=I;
%      Z(I)=L*c(N+1:N+m); N=N+m;
   end;
  case 9
% if k==9
[lam1,lam2,lam3,lam4,lam5,lam6,lam7,lam8,lam9] = ... 
bary9(V(p(1),:),V(p(2),:),V(p(3),:),V(p(4),:),V(p(5),:),V(p(6),:),V(p(7),:),V(p(8),:),V(p(9),:),X,Y);
I=find(lam1>=-tol&lam2>= -tol&lam3>= -tol&lam4>=-tol&lam5>=-tol&lam6>=-tol &lam7>=-tol&lam8>=-tol&lam9>=-tol);
   if ~isempty(I)
       v=[V(p(1),:);V(p(2),:);V(p(3),:);V(p(4),:);V(p(5),:);V(p(6),:);V(p(7),:);V(p(8),:);V(p(9),:)];
       xx=X(I);yy=Y(I); 
      L = mlag2dDeg2(v,[xx,yy],type);
           M2{j}=L; S2{j}=I;
%      Z(I)=L*c(N+1:N+m); N=N+m;
   end
   case 10
% if k==10
[lam1,lam2,lam3,lam4,lam5,lam6,lam7,lam8,lam9,lam10] = ... 
bary10(V(p(1),:),V(p(2),:),V(p(3),:),V(p(4),:),V(p(5),:),V(p(6),:),V(p(7),:),V(p(8),:),V(p(9),:),V(p(10),:),X,Y);
I=find(lam1>=-tol&lam2>= -tol&lam3>= -tol&lam4>=-tol&lam5>=-tol&lam6>=-tol ...
    &lam7>=-tol&lam8>=-tol&lam9>=-tol &lam10>=-tol);
   if ~isempty(I)
       v=[V(p(1),:);V(p(2),:);V(p(3),:);V(p(4),:);V(p(5),:);V(p(6),:);V(p(7),:);V(p(8),:);V(p(9),:);V(p(10),:)];
       xx=X(I);yy=Y(I); 
      L = mlag2dDeg2(v,[xx,yy],type);
           M2{j}=L; S2{j}=I;
%      Z(I)=L*c(N+1:N+m); N=N+m;
   end;
end;
end
N=0;
for j=1:n
    m=M1(j);
    L=M2{j}; I=S2{j};
     Z(I)=L*c(N+1:N+m); N=N+m;
end




function nL = mlag2dDeg2(v,x,type)
%
% Evaluate generalized quadratic Lagrange functions
% and gradients in a convex polygon
%
% Inputs:
% v    : [x1 y1; x2 y2; ...; xn yn], the n vertices of the polygon, ccw
% x    : [x(1) x(2)], the point at which the functions are evaluated
% type : type of barycentric coordinate:
%        type=1 => Wachspress
%        type=2 => MV local
%        type=3 => bilinear, if n=4.
%        type=4 => family.
% Outputs:
% L  : output basis functions = [L_11; L_12; L_22; ...; L_2n+1]
% dL : output gradient of basis functions = [dL_11; dL_12; ...; dL_2n+1]
%
% Author: Ming-Jun Lai and Michael Floater 
% e-mail: mjlai@math.uga.edu
% Created: Dec. 2014, using Matlab R2014b
%This is based on the results in the publication:
%  Polygonal spline spaces and  the numerical solution of the Poisson equation,
%    Michael Floater and Ming-Jun Lai
%  SIAM J. Numerical Analysis, accepted on  2015.
%

  m=size(x,1);
  n = size(v,1);
  L = zeros(2*n,m);
  nL=[];

 % for j=1:m
 %   xx=x(j,:); 
    B = BB2dDeg2Pn(v,x,type);
    for i=1:n
      im1 = mod1(i-1,n);
      L(2*i-1,:) = B(2*i-1,:) - 0.5 * (B(2*im1,:)+B(2*i,:));
      L(2*i,:)   = 2*B(2*i,:);
    end
    nL=L';
%    nL=[nL;L'];


