function [E,PE,PV,EV,BV] = vdata(V,P);
%        [E,PE,PV,EV,BV] = vdata(V,P)
% Input = Vertices and polygons of a partition. 
% Output:
% Edges = a list of edges
% PE = Num(polygons) X Num(edges) matrix whose (i,j)th entry 
% = 1 if edge #j is an edge for polygon #i.
% PV = Num(polygons) X Num(vertices) matrix whose (i,j)th entry
% = 1 if vertex #j is a vertex for polygon #i.
% EV = Num(edges) X Num(vertices) matrix whose (i,j)th entry
% = 1 if edge #i has vertex #j as an endpoint.
% BV = a list, in no particular order, of boundary vertices
% This is written by Dr. Ming-Jun Lai on Nov. 5, 2014.
numPoly = size(P,2);
E = [];
PE = [];
numEdges = 0;
for i = 1:numPoly
    polygon = P{i};
    vertsInPoly = size(polygon,2);
    for j = 1:vertsInPoly
        edge = [min([polygon(j) polygon(rem(j,vertsInPoly)+1)]),...
             max([polygon(j) polygon(rem(j,vertsInPoly)+1)])];
        if ~isempty(E)
            edgenum = find((edge(1) == E(:,1))&(edge(2) == E(:,2))); 
        else
            edgenum = [];
        end;
        if isempty(edgenum) 
            E = [E;edge];
            numEdges = numEdges + 1; 
            PE = [PE sparse(numPoly,1)];
            edgenum = numEdges;
        end;
        PE(i,edgenum) = 1;
    end;
end;
numV = size(V,1); 
PV = sparse(numPoly,numV);
for i = 1:numPoly
    PV(i,P{i}) = ones(1,size(P{i},2));
end; 
EV = sparse(numEdges,numV);
for i = 1:numEdges
    EV(i,E(i,:)) = ones(1,2);
end;

BE = find(sum(PE)==1);
BV = [];
for i=1:size(BE,2)
	BV = [BV E(BE(i),1) E(BE(i),2)];
end
BV = unique(BV);