function [phi dxphi dyphi] = wachspress2dP(v,x)
%
% Evaluate Wachspress basis functions and their gradients in a convex polygon
%
% Inputs:
% v    : [x1 y1; x2 y2; ...; xn yn], the n vertices of the polygon in ccw
% x    : [x(1) x(2)], the point at which the basis functions are computed
% Outputs:
% phi  : output basis functions = [phi_1; ...; phi_n]
% dphi : output gradient of basis functions = [dphi_1; ...; dphi_n]
%
% Authors: Michael Floater and Ming-Jun Lai
% e-mail: michaelf@ifi.uio.no and mjlai@uga.edu
% Created: June 2013, using Matlab R2013b
% Modified: Mach, 2015.
% Copyright 2015
m=size(x,1);
n = size(v,1);
w = zeros(n,m);
R = zeros(n,2);
phi  = zeros(n,m);
dxphi = zeros(n,m); dyphi=zeros(n,m);
un = getNormals(v);
myun1=ones(n,m); myun2=ones(n,m);
p1 = zeros(n,m); p2=p1; h=zeros(1,m);
for i = 1:n
%  h = dot(v(i,:) - x,un(i,:));
  myun1(i,:)=un(i,1)*myun1(i,:);
  myun2(i,:)=un(i,2)*myun2(i,:);
   h = (v(i,1) - x(:,1)').*myun1(i,:) + (v(i,2)- x(:,2)').*myun2(i,:);
  p1(i,:)= myun1(i,:)./h;
  p2(i,:)= myun2(i,:)./h;
end
w=zeros(n,m);
for i = 1:n
  im1 = mod(i-2,n) + 1;
%  w(i) = det([p(im1,:);p(i,:)]);
  w(i,:) = p1(im1,:).*p2(i,:) - p2(im1,:).*p1(i,:);
  R1(i,:) = p1(im1,:) + p1(i,:); 
  R2(i,:) = p2(im1,:) + p2(i,:); 
end
for i=1:n
phi(i,:) = w(i,:)./sum(w);
end

phiR1 = sum(phi .* R1); 
phiR2 = sum(phi .* R2);
for i = 1:n
  dxphi(i,:) = phi(i,:).* (R1(i,:) - phiR1(1,:));
  dyphi(i,:) = phi(i,:).* (R2(i,:) - phiR2(1,:));
end
